import "dart:html";
import "dart:svg" hide Point;
import "dart:math";
import "package:hexgrid/hexgrid.dart";

class TilePath {
  List<Point> _corners;
  String _fill;

  get path {
    var d = _corners.fold("M${_corners.last.x} ${_corners.last.y}",
      (sofar, next) => sofar + " L${next.x} ${next.y}");
    var g = new GElement();
    var p = new PathElement()
      ..setAttribute("class", "hexoutline")
      ..setAttribute("d", d)
      //..setAttribute("stroke", "Green")
      //..setAttribute("stroke-width", "1px")
      ..setAttribute("fill", _fill)
      ..setAttribute("sp-origFill", _fill);
    g.append(p);
    return g;
  }

  TilePath(this._corners, this._fill);
}

class Tile {
  Hex h;
  String fill;

  outline(Layout layout) => new TilePath(h.corners(layout), fill);

  Tile neighbor(int i) => new Tile(h.neighbor(i));

  Tile(this.h, [this.fill="DarkCyan"]);
}

draw_tile(Element host, Layout layout, Tile tile) {
    var outline = tile.outline(layout);
    host.append(outline.path);
}

main() {
  var host = querySelector("#hexgrid1"),
      layout_size = new Point(20, 20),
      layout_origin = new Point(20, 20),
      layout = new Layout.orient_pointy(layout_size, layout_origin),
      start = new Tile(new Hex(0, 0)),
      tile,
      rows = 15,
      cols = 30,
      random = new Random(),
      tiles = new List(),
      yes;

  var falseLimit = 1,
      forceTrue = false,
      falseCount = 0,
      nextNeighbor = 5;

  for (int row=0; row<rows; row++) {
    tile = start;
    for (int col=0; col<cols; col++) {
      if (forceTrue ||
          col == 0 || col == cols-1 ||
          row == 0 || row == rows-1) {
        draw_tile(host, layout, tile);
        if (forceTrue) falseCount--;
        forceTrue = falseCount > 0;
      } else {
        yes = random.nextBool();
        if (yes) {
          draw_tile(host, layout, tile);
        } else {
          falseCount += 5;
          forceTrue = falseCount >= falseLimit;
        }
      }
      tile = tile.neighbor(0);
    }
    start = start.neighbor(nextNeighbor);
    nextNeighbor = nextNeighbor == 5 ? 4 : 5;
  }

  querySelectorAll(".hexoutline")
    ..onMouseOver.listen((MouseEvent e) {
      //var h = new Hex.fromPoint(layout, e.offset - layout_origin);
      e.target.setAttribute("fill", "Cyan");
    })
    ..onMouseOut.listen((MouseEvent e) {
      e.target.setAttribute("fill", e.target.getAttribute("sp-origFill"));
    });
}